/* connectsock.c - connectsock */

#define _POSIX_C_SOURCE 200112L				/*	or #define _POSIX_C_SOURCE 200809L	*/

#include <sys/socket.h>

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <err.h>
#include <errno.h>

/*-----------------------------------------------------------------------
 * connectsock - allocate & connect a socket using TCP or UDP            
 *-----------------------------------------------------------------------*/
int
connectsock(const char *host, const char *service, const char *transport )
/*
 * Arguments:
 *      host      - name of host to which connection is desired
 *      service   - service associated with the desired port
 *      transport - name of transport protocol to use ("tcp" or "udp")
 */
{
	struct addrinfo *res, *resptr;
	struct addrinfo hints = { 0 };
	int ret;
	int connsd, type; /* socket descriptor and socket type */

	/* Use protocol to choose a socket type */
	if (strcmp(transport, "UDP") == 0)
		type = SOCK_DGRAM;
	else if (strcmp(transport, "TCP") == 0)
		type = SOCK_STREAM;
	else
		errx(1, "connectsock arguments error: unknown transport\n");

	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = type;

	if ((ret = getaddrinfo(host, service, &hints, &res)) != 0)
		errx(1, "getaddrinfo: %s\n", gai_strerror(ret));
	
	for (resptr = res; resptr != NULL; resptr = resptr->ai_next) {
		/* Allocate a socket */
		if ((connsd = socket(resptr->ai_family, resptr->ai_socktype, resptr->ai_protocol)) < 0) {
			ret = errno;
		}
		/* Connect the socket */
		else if ((connect(connsd, resptr->ai_addr, resptr->ai_addrlen)) < 0) {
			ret = errno;
			close(connsd);
		}
		else {
			freeaddrinfo(res);
			return connsd;
		}	
	}
	errno = ret;
	errx(1, "can't connect to %s.%s: %s\n", host, service, strerror(errno));
}

/**************************************************************************
 * Building a shared library(DLLs):                                       *
 * gcc -Wall -pedantic -std=c11 -g -shared -fpic                          *
 *     -o connectsock.dll connectsock.c connectTCP.c connectUDP.c         *
 *                                                                        *
 *                                                                        *
 * To link:                                                               *
 * -L ../api -l connectsock                                               *
 *                                                                        *
 * To run:                                                                *
 * cp connectsock.dll ../apps/connectsock.dll                             *
 **************************************************************************/