/* TCPdaytime.c - main, TCPdaytime */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <err.h>

int	TCPdaytime(const int);
int	connectTCP(const char *, const char *);

#define	LINELEN		4096

/*-----------------------------------------------------------------------
 * main - TCP client for DAYTIME service
 *-----------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	const char *host = "localhost";	/* host to use if none supplied	*/
	const char *service = "daytime";	/* default service port		*/
	int sockd;	/* socket descriptor */	

	switch (argc) {
	case 1:
		break;
	case 3:
		service = argv[2];
		/* FALL THROUGH */
	case 2:
		host = argv[1];
		break;
	default:
		errx(0, "usage: %s [host [port]]\n",  argv[0]);
	}
	
	sockd = connectTCP(host, service);	
	exit(TCPdaytime(sockd));
}

/*-----------------------------------------------------------------------
 * TCPdaytime - Do TCP Daytime on specified socket and print results
 *-----------------------------------------------------------------------
 */
int
TCPdaytime(const int sd)
{
	char	buf[LINELEN+1];		/* buffer for one line of text	*/
	ssize_t	nchars;			/* read count*/

	while((nchars = read(sd, buf, LINELEN))) {
		if (nchars < 0) 
			err(1, "TCPdaytime() read");
		
		buf[nchars] = '\0';		/* ensure null-terminated	*/
		if (fputs(buf, stdout) == EOF)
			err(1, "TCPdaytime() fputs");
	}
	
	return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPdaytime TCPdaytime.c -L../../api -lconnectsock   *
 * To run:                                                               *
 * ../../apps/TCPdaytime 127.0.0.1                                       *
 * ../../apps/TCPdaytime 0::1                                            *
 *************************************************************************/