/* TCPdaytimed.c - TCPdaytimed, main */

#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <locale.h>
#include <langinfo.h>

void	TCPdaytimed(const int);
int		passiveTCP(const char *, const int);
const char *satop(const struct sockaddr *, char *, socklen_t);

#define QLEN	10

/*------------------------------------------------------------------------
 * main - Iterative TCP server for DAYTIME service
 *------------------------------------------------------------------------
 */

int
main(int argc, char *argv[])
{
	const char *service = "daytime"; /* service name or port number */
	int	msock, ssock; /* master & slave sockets */
	struct	sockaddr_storage sstorage; /* the from address of a client */
	socklen_t salen; /* from-address length */
	char fmtaddr[128]; /* the formated address string */

	switch (argc) {
	case	1:
		break;
	case	2:
		service = argv[1];
		break;
	default:
		errx(0, "usage: %s [port]\n",  argv[0]);
	}

	msock = passiveTCP(service, QLEN);

	while (1) {
		salen = sizeof sstorage;
		ssock = accept(msock, (struct sockaddr *)&sstorage, &salen);
		if (ssock < 0)
			err(1, "main() accept");

        if ((satop((struct sockaddr *)&sstorage, fmtaddr, sizeof fmtaddr)) == NULL)
        	err(1, "main() satop failed");
    	printf("connection from %s\n", fmtaddr);

		TCPdaytimed(ssock);
		(void) close(ssock);
	}
}

/*------------------------------------------------------------------------
 * TCPdaytimed - do TCP DAYTIME protocol
 *------------------------------------------------------------------------
 */

void
TCPdaytimed(const int sd)
{
	char	dts[256];	/* buffer for datetime string */
	time_t	now;		/* current time */

	(void)setlocale(LC_ALL, "");
	(void) time(&now);
	(void)strftime(dts, sizeof dts, nl_langinfo(D_T_FMT), localtime(&now));
	if (write(sd, dts, strlen(dts)) != (ssize_t)strlen(dts))
		err(1, "TCPdaytimed() write");
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPdaytimed TCPdaytimed.c -L../../api -lpassivesock *
 *                                                                       *
 * To run:                                                               *
 * ../../apps/TCPdaytimed                                                *
 *************************************************************************/