/* UDPdaytimed.c - UDPdaytimed, main */

#include <netinet/in.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <locale.h>
#include <langinfo.h>
#include <stdio.h>

int passiveUDP(const char *);
void UDPdaytimed(const int);
const char *satop(const struct sockaddr *, char *, socklen_t);

#define UNIXEPOCH 2208988800UL /* UNIX epoch, in UCT secs	*/

/*------------------------------------------------------------------------
 * main - Iterative UDP server for DAYTIME service
 *------------------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	const char *service = "daytime"; /* service name or port number	*/
	int sock; /* server socket		*/

	switch (argc)
	{
	case 1:
		break;
	case 2:
		service = argv[1];
		break;
	default:
		errx(0, "usage: %s [port]\n", argv[0]);
	}

	sock = passiveUDP(service);
	while (1)
	{
		UDPdaytimed(sock);
	}
}

/*------------------------------------------------------------------------
 * UDPdaytimed - do UDP DAYTIME on the given socket
 *------------------------------------------------------------------------
 */

void UDPdaytimed(const int sd)
{
	struct sockaddr_storage fsa; /* the from address of a client */
	socklen_t salen;				/* from-address length */
	char buf[1];				/* "input" buffer; any size > 0	*/
	time_t now;					/* current time */
	char dts[256];				/* buffer for datetime string */
	char fmtaddr[128];			/* the formated address string */
	ssize_t	nchars; 			/* receive count */

	salen = sizeof fsa;
	if ((nchars = recvfrom(sd, buf, sizeof buf, 0,
				 (struct sockaddr *)&fsa, &salen)) > 0) {
    	if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
        	err(1, "UDPdaytimed() satop failed");
    	printf("data from %s\n", fmtaddr);

		(void)setlocale(LC_ALL, "");
		(void)time(&now);
		(void)strftime(dts, sizeof dts, nl_langinfo(D_T_FMT), localtime(&now));
		if (sendto(sd, dts, strlen(dts), 0,
			(struct sockaddr *)&fsa, sizeof fsa) != (ssize_t)strlen(dts))
			err(1, "UDPdaytimed() sendto");
	}
	if (nchars < 0) err(1, "UDPdaytimed() recvfrom");
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/UDPdaytimed UDPdaytimed.c -L../../api -lpassivesock *
 *                                                                       *
 * To run:                                                               *
 * ../../apps/UDPdaytimed                                                *
 *************************************************************************/