/* daytimed.c - main, daytime */

#include <sys/select.h>
#include <netinet/in.h>
#include <unistd.h>

#include <string.h>
#include <time.h>
#include <err.h>
#include <stdio.h>
#include <errno.h>
#include <locale.h>
#include <langinfo.h>

int daytime(char *, size_t);
int passiveTCP(const char *, const int);
int passiveUDP(const char *);
const char *satop(const struct sockaddr *, char *, socklen_t);

#ifndef MAX
#define MAX(x, y) ((x) > (y) ? (x) : (y))
#endif /* MAX */

#define QLEN 10
#define LINELEN 4096

/*------------------------------------------------------------------------
 * main - Iterative server for DAYTIME service
 *------------------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	const char *service = "daytime";	 /* service name or port number	*/
	char buf[256];				 /* buffer for datetime string */
	struct sockaddr_storage fsa; /* the from address of a client */
	socklen_t salen;				 /* from-address length		*/
	int tsock;					 /* TCP master socket		*/
	int usock;					 /* UDP socket			*/
	int nfds;
	fd_set rfds; /* readable file descriptors	*/
    char fmtaddr[128];			/* the formated address string */
	ssize_t	nchars; /* receive count */	

	switch (argc)
	{
	case 1:
		break;
	case 2:
		service = argv[1];
		break;
	default:
		errx(0, "usage: %s [port]\n", argv[0]);
	}

	tsock = passiveTCP(service, QLEN);
	usock = passiveUDP(service);
	nfds = MAX(tsock, usock) + 1; /* bit number of max fd	*/

	FD_ZERO(&rfds);

	while (1)
	{
		FD_SET(tsock, &rfds);
		FD_SET(usock, &rfds);

		if (select(nfds, &rfds, (fd_set *)0, (fd_set *)0,
				   (struct timeval *)0) < 0)
			err(1, "main() select");
		if (FD_ISSET(tsock, &rfds))
		{
			int ssock; /* TCP slave socket	*/

			salen = sizeof fsa;
			ssock = accept(tsock, (struct sockaddr *)&fsa,
						   &salen);
			if (ssock < 0)
				err(1, "main() accept");

			if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
				err(1, "main() satop failed");
			printf("daytime connection from %s\n", fmtaddr);

			daytime(buf, sizeof buf);

			if (write(ssock, buf, strlen(buf)) != (ssize_t)strlen(buf))
				err(1, "main() write");
			(void)close(ssock);
		}
		if (FD_ISSET(usock, &rfds))
		{
			salen = sizeof fsa;
			if ((nchars = recvfrom(usock, buf, sizeof buf, 0,
						 (struct sockaddr *)&fsa, &salen)) > 0) {
				if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
					err(1, "main() satop failed");
				printf("daytime request from %s\n", fmtaddr);

				daytime(buf, sizeof buf);
				if (sendto(usock, buf, strlen(buf), 0,
					   (struct sockaddr *)&fsa, sizeof fsa) != (ssize_t)strlen(buf))
					err(1, "main() sendto faile");
			}
			if (nchars < 0) err(1, "main() recvfrom");
		}
	}
}

/*------------------------------------------------------------------------
 * daytime - fill the given buffer with the time of day
 *------------------------------------------------------------------------
 */
int daytime(char *dts, size_t size)
{
	time_t now;
	(void)setlocale(LC_ALL, "");
	(void)time(&now);
	(void)strftime(dts, size, nl_langinfo(D_T_FMT), localtime(&now));
	return 0;
}

/************************************************************************
 * To compile:                                                          *
 * gcc -Wall -pedantic -std=c11 -g                                      *
 *     -o ../../apps/daytimed daytimed.c -L../../api -lpassivesock      *
 *                                                                      *
 * To run:                                                              *
 * ../../apps/daytimed                                                  *
 ************************************************************************/