/* TCPecho.c - main, TCPecho */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <err.h>

int	TCPecho(const int);
int	connectTCP(const char *, const char *);

#define	LINELEN		4096

/*------------------------------------------------------------------------
 * main - TCP client for ECHO service
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	const char *host = "localhost";	/* host to use if none supplied	*/
	const char *service = "echo";	/* default service name		*/
	int sockd; /* socket descriptor */	

	switch (argc) {
	case 1:
		break;
	case 3:
		service = argv[2];
		/* FALL THROUGH */
	case 2:
		host = argv[1];
		break;
	default:
		errx(0, "usage: %s [host [port]]\n",  argv[0]);
	}

	sockd = connectTCP(host, service);	
	exit(TCPecho(sockd));
}

/*------------------------------------------------------------------------
 * TCPecho - do TCP ECHO on specified socket and print reply
 *------------------------------------------------------------------------
 */
int
TCPecho(const int sd)
{
	char	*lineptr = NULL;  /* pointer to the buffer containing one line of text */
    size_t  len = 0;
    ssize_t	inchars, outchars, nchars;  /* characters sent and received */

	while ( (outchars = getline(&lineptr, &len, stdin)) != -1) {
		/* send input to ECHO service on specified socket */
		if (write(sd, lineptr, (size_t)outchars) != outchars)
			err(1, "TCPecho() write");

		/* read it back */
		for (inchars = 0; inchars < outchars; inchars += nchars) {
			if ((nchars = read(sd, lineptr + inchars, (size_t)(outchars - inchars))) < 0) 
				err(1, "TCPecho() read");
			else if (nchars == 0)
				break;
		}

		if (inchars > 0) {
			*(lineptr + inchars) = '\0';	/* insure line null-terminated	*/
			if (fputs(lineptr, stdout) == EOF)
				err(1, "TCPecho() fputs");
		}
	}

	if ((outchars == -1 ) && (ferror(stdin)))
		err(1, "UDPecho() getline");
	
	if (lineptr != NULL)
    	free(lineptr);
	return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPecho TCPecho.c -L../../api -lconnectsock         *
 * To run:                                                               *
 * ../../apps/TCPecho 127.0.0.1                                          *
 * ../../apps/TCPecho 0::1                                               *
 *************************************************************************/