/* TCPechod.c - main, TCPechod */

#include <sys/wait.h>
#include <netinet/in.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>

#define	QLEN		  32	/* maximum connection queue length	*/
#define	BUFSIZE		8192

void	reaper(int);
int	TCPechod(const int);
int	passiveTCP(const char *, const int);
const char *satop(const struct sockaddr *, char *, socklen_t);

/*------------------------------------------------------------------------
 * main - Concurrent TCP server for ECHO service
 *------------------------------------------------------------------------
 */

int
main(int argc, char *argv[])
{
	const char *service = "echo";	/* service name or port number	*/
	struct sockaddr_storage fsa;	/* the from address of a client */
	socklen_t salen;	/* from-address length */
	int	msock;			/* master server socket		*/
	int	ssock;			/* slave server socket		*/
	char fmtaddr[128];	/* the formated address string */

	switch (argc) {
	case	1:
		break;
	case	2:
		service = argv[1];
		break;
	default:
		errx(0, "usage: %s [port]\n",  argv[0]);
	}

	msock = passiveTCP(service, QLEN);

	(void) signal(SIGCHLD, reaper);

	while (1) {
		salen = sizeof fsa;
		ssock = accept(msock, (struct sockaddr *)&fsa, &salen);
		if (ssock < 0) {
			if (errno == EINTR)
				continue;
			err(1, "main() accept");
		}

        if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
        	err(1, "main() satop failed");
    	printf("connection from %s\n", fmtaddr);

		switch (fork()) {
		case 0:		/* child */
			(void) close(msock);
			exit(TCPechod(ssock));
		default:	/* parent */
			(void) close(ssock);
			break;
		case -1:
			err(1, "main() fork");
		}
	}
}

/*------------------------------------------------------------------------
 * TCPechod - echo data until end of file
 *------------------------------------------------------------------------
 */

int
TCPechod(const int sd)
{
	char	buf[BUFSIZE];
	ssize_t		nchars;

	while ((nchars = read(sd, buf, sizeof buf)) > 0)
		if (write(sd, buf, (size_t)nchars) != nchars)
			err(1, "TCPechod() write");

	if (nchars < 0)
		err(1, "TCPechod() read");

	return 0;
}

/*------------------------------------------------------------------------
 * reaper - clean up zombie children
 *------------------------------------------------------------------------
 */

void
reaper(int sig)
{
	int	status;

	while (wait3(&status, WNOHANG, (struct rusage *)0) >= 0)
		/* empty */;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPechod TCPechod.c -L../../api -lpassivesock       *
 *                                                                       *
 * To run:                                                               *
 * ../../apps/TCPechod                                                    *
 *************************************************************************/