/* TCPmechod.c - main, echo */

#include <sys/select.h>
#include <netinet/in.h>
#include <unistd.h>
#include <string.h>
#include <err.h>
#include <stdio.h>

#define QLEN 10 /* maximum connection queue length */
#define BUFSIZE 8192

int passiveTCP(const char *, const int);
const char *satop(const struct sockaddr *, char *, socklen_t);
int echo(const int);

/*------------------------------------------------------------------------
 * main - Concurrent TCP server for ECHO service
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
    const char *service = "echo"; /* service name or port number */
    struct sockaddr_storage fsa; /* the from address of a client */
    socklen_t salen; /* from-address length */
    int msock; /* master server socket */
    fd_set rfds; /* read file descriptor set */
    fd_set afds; /* active file descriptor set */
    int sd, nfds;
    char fmtaddr[128];			/* the formated address string */

    switch (argc)
    {
    case 1:
        break;
    case 2:
        service = argv[1];
        break;
    default:
        errx(0, "usage: %s [port]\n",  argv[0]);
    }

    msock = passiveTCP(service, QLEN);

    nfds = FD_SETSIZE;
    FD_ZERO(&afds);
    FD_SET(msock, &afds);

    while (1)
    {
        memcpy(&rfds, &afds, sizeof rfds);

        if (select(nfds, &rfds, (fd_set *)0, (fd_set *)0,
                   (struct timeval *)0) < 0)
            err(1, "main() select");
        if (FD_ISSET(msock, &rfds))
        {
            int ssock;

            salen = sizeof fsa;
            ssock = accept(msock, (struct sockaddr *)&fsa,
                           &salen);
            if (ssock < 0)
                err(1, "main() accept");

            if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
        	    err(1, "main() satop failed");
    	    printf("echo connection from %s\n", fmtaddr);

            FD_SET(ssock, &afds);
        }
        for (sd = 0; sd < nfds; ++sd)
            if (sd != msock && FD_ISSET(sd, &rfds))
                if (echo(sd) == 0)
                {
                    (void) close(sd);
                    FD_CLR(sd, &afds);
                }
    }
}

/*------------------------------------------------------------------------
 * echo - echo one buffer of data, returning byte count
 *------------------------------------------------------------------------
 */
int
echo(const int sd)
{
    char buf[BUFSIZE];
    ssize_t cc;

    cc = read(sd, buf, sizeof buf);
    if (cc < 0)
        err(1, "echo() read");
    if (cc && (write(sd, buf, (size_t)cc) != cc) )
        err(1, "echo() write");

    return cc;
}

/************************************************************************
 * To compile:                                                          *
 * gcc -Wall -pedantic -std=c11 -g                                      *
 *     -o ../../apps/TCPmechod TCPmechod.c -L../../api -lpassivesock    *
 *                                                                      *
 * To run:                                                              *
 * ../../apps/TCPmechod                                                 *
 ************************************************************************/