/* TCPtechod.c - main, TCPechod */

#include <netinet/in.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include <errno.h>
#include <pthread.h>


#define QLEN 10             /* maximum connection queue length	*/
#define BUFSIZE 4096

int TCPechod(const int);
int passiveTCP(const char *, const int);
const char *satop(const struct sockaddr *, char *, socklen_t);

/*------------------------------------------------------------------------
 * main - Concurrent TCP server for ECHO service
 *------------------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	pthread_t th;
	pthread_attr_t ta;
	const char *service = "echo";		 /* service name or port number	*/
	struct sockaddr_storage fsa; /* the address of a client     */
	socklen_t salen;				 /* length of client's address	*/
	int msock;					 /* master server socket		*/
	int ssock;					 /* slave server socket         */
	char fmtaddr[128];			/* the formated address string */

	switch (argc)
	{
	case 1:
		break;
	case 2:
		service = argv[1];
		break;
	default:
		errx(0, "usage: %s [port]\n", argv[0]);
	}

	msock = passiveTCP(service, QLEN);

	(void)pthread_attr_init(&ta);
	(void)pthread_attr_setdetachstate(&ta, PTHREAD_CREATE_DETACHED);

	while (1)
	{
		salen = sizeof fsa;
		ssock = accept(msock, (struct sockaddr *)&fsa, &salen);
		if (ssock < 0)
		{
			if (errno == EINTR)
				continue;
			err(1, "main() accept");
		}

        if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
        	err(1, "main() satop failed");
    	printf("echo connection from %s\n", fmtaddr);

		if (pthread_create(&th, &ta, (void *(*)(void *))TCPechod,
						   (void *)(long)ssock) < 0)
			err(1, "main() pthread_create");
	}
}

/*------------------------------------------------------------------------
 * TCPechod - echo data until end of file
 *------------------------------------------------------------------------
 */
int TCPechod(const int sd)
{
	char buf[BUFSIZE];
	ssize_t cc;

	while ((cc = read(sd, buf, sizeof buf)) > 0)
		if (write(sd, buf, (size_t)cc) != cc)
			err(1, "TCPechod() write");
	if (cc < 0)
		err(1, "TCPechod() read");

	return 0;
}


/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPtechod TCPtechod.c -L../../api -lpassivesock     *
 *                                                                       *
 * To run:                                                               *
 * ../../apps/TCPtechod                                                  *
 *************************************************************************/