/* UDPecho.c - main, UDPecho */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <err.h>

int	UDPecho(const int);
int	connectUDP(const char *, const char *);

#define	LINELEN		4096

/*------------------------------------------------------------------------
 * main - UDP client for ECHO service
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	const char *host = "localhost";
	const char *service = "echo";
	int sockd; /* socket descriptor */

	switch (argc) {
	case 1:
		break;
	case 3:
		service = argv[2];
		/* FALL THROUGH */
	case 2:
		host = argv[1];
		break;
	default:
		errx(0, "usage: %s [host [port]]\n",  argv[0]);
	}

	sockd = connectUDP(host, service);
	exit(UDPecho(sockd));
}

/*------------------------------------------------------------------------
 * UDPecho - do UDP ECHO on specified socket and print reply
 *------------------------------------------------------------------------
 */
int
UDPecho(const int sd)
{
	char	*lineptr = NULL;	/* pointer to the buffer containing one line of text */
    size_t  len = 128;
    ssize_t	inchars, outchars;			/*  characters sent and received */

	while ( (outchars = getline(&lineptr, &len, stdin)) != -1) {
		/* send input to ECHO service on specified socket */
		if (write(sd, lineptr, (size_t)outchars) != outchars)
			err(1, "TCPecho() write");
		/* read it back */
		if ((inchars = read(sd, lineptr, len - 1)) > 0) {
			*(lineptr + inchars) = '\0';	/* insure null-terminated */
			if (fputs(lineptr, stdout) == EOF)
				err(1, "UDPecho() fputs");
		}

		if (inchars < 0) err(1, "UDPecho() read");
	}
	if ( (outchars == -1) && (ferror(stdin) ) )
		err(1, "UDPecho() getline");

	if (lineptr != NULL)
    	free(lineptr);
	return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/UDPecho UDPecho.c -L../../api -lconnectsock         *
 * To run:                                                               *
 * ../../apps/UDPecho 127.0.0.1                                          *
 * ../../apps/UDPecho 0::1                                               *
 *************************************************************************/