/* TCPhellod3.c - main */

#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <err.h>

int passiveTCPsock(const char *);
int TCPhellod(const int);

#define QLEN    10

/*------------------------------------------------------------------------
 * main - Iterative TCP(IPv6) server for Hello World!
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{ 
	const char *service = "5678";	/* port number	*/
	int listensockd; /* listen socket descriptor */
    int connectionsockd; /* connection establishment socket descriptor */
    struct sockaddr_in6 fsa; /* the from address of a client */
    socklen_t salen; /* from-address length */
    char fmtaddr[128]; /* the formated address string */

    /* 创建用于监听连接请求的套接字listensockd */
    listensockd = passiveTCPsock(service);
    
    /* 迭代 */
    while (1) {  	
        /* 接受下一连接请求 */
        salen = sizeof fsa;
        connectionsockd = accept(listensockd, (struct sockaddr *)&fsa, &salen);

        inet_ntop(AF_INET6, &fsa.sin6_addr.s6_addr, fmtaddr, sizeof fmtaddr);
        printf("connection from [%s]:%d\n", fmtaddr, ntohs(fsa.sin6_port));
  
        /* 在成功建立连接的套接字(connectionsockd)上发送Hello应答 */
        (void) TCPhellod(connectionsockd);

        /* 关闭连接 */
        (void) close(connectionsockd);
    }
}

int
passiveTCPsock(const char *service)
{
    struct sockaddr_in6 sin6; /* an Inernet endpoint address */
	int pasvsd; /* passive socket descriptor */

    /* initialize sockaddr_in6 structure */
    memset(&sin6, 0, sizeof sin6);
    sin6.sin6_family = AF_INET6;
    sin6.sin6_addr = in6addr_any;
    sin6.sin6_port = htons((uint16_t)atoi(service));

    /* create a socket */
    pasvsd = socket(PF_INET6, SOCK_STREAM, 0);

    /* bind the socket */
    bind(pasvsd, (struct sockaddr *)&sin6, sizeof sin6);

    /* listen to the socket */
    listen(pasvsd, QLEN);

    return pasvsd;
}

int
TCPhellod(const int tsockd)
{  
    char replymsg[] = "Hello World!\n";
    /* send the hello message */
    send(tsockd, replymsg, strlen(replymsg), 0);

    return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/TCPipv6hellod.exe TCPipv6hellod.c                   *
 * To run:                                                               *
 * ../apps/TCPipv6hellod.exe                                             *
 *                                                                       *
 *************************************************************************/