/* UDPhellod3.c - main */

#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <err.h>

int passiveUDPsock(const char *);
int UDPhellod(const int);

/*------------------------------------------------------------------------
 * main - Iterative UDP(IPv6) server for Hello World!
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
    const char *service = "5678"; /* service port number	*/
    int servsockd; /* socket descriptor */
   
    /* 创建hello服务器（UDP）套接字（servsockd） */
    servsockd = passiveUDPsock(service);
    
    /* 迭代 */
	while (1) {
        /* 接收请求并进行处理，构造并发送应答 */
        (void) UDPhellod(servsockd);
	}
}

int
passiveUDPsock(const char *service)
{
    struct sockaddr_in6 sin6; /* an Inernet endpoint address */
	int pasvsd; /* passive socket descriptor */

    /* initialize sockaddr_in6 structure */
    memset(&sin6, 0, sizeof sin6);
    sin6.sin6_family = AF_INET6;
    sin6.sin6_addr = in6addr_any;
    sin6.sin6_port = htons((uint16_t)atoi(service));

    /* create a socket */
    pasvsd = socket(PF_INET6, SOCK_DGRAM, 0);

    /* bind the socket */
    bind(pasvsd, (struct sockaddr *)&sin6, sizeof sin6);

    return pasvsd;
}

int UDPhellod(const int usockd)
{
    char replymsg[] = "Hello World!\n";
    struct sockaddr_in6 fsa; /* the from socket address of a client */
	socklen_t salen; /* from-address length */
    char buf[1]; /* "input" buffer; any size > 0 */
    char fmtaddr[128]; /* the formated address string */

	salen = sizeof fsa;
    /* receive next request */
	recvfrom(usockd, buf, sizeof buf, 0, 
                (struct sockaddr *)&fsa, &salen);
    inet_ntop(AF_INET6, &fsa.sin6_addr.s6_addr, fmtaddr, sizeof fmtaddr);
    printf("data from [%s]:%d\n", fmtaddr, ntohs(fsa.sin6_port));

    /* handle the request, formulate a reply */

    /* send the reply */
    sendto(usockd, replymsg, strlen(replymsg), 0, 
            (struct sockaddr *)&fsa, sizeof fsa);

	return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/UDPipv6hellod.exe UDPipv6hellod.c                   *
 * To run:                                                               *
 * ../../apps/UDPipv6hellod.exe                                          *
 *                                                                       *
 *************************************************************************/