/* superd.c - main */

#include <netinet/in.h>
#include <sys/select.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include <errno.h>
#include <sys/wait.h>

#define	UDP_SERV	0
#define	TCP_SERV	1

#define	NOSOCK		-1	/* an invalid socket descriptor	*/

struct service {
	const char *sv_name;
	char sv_useTCP;
	int sv_sock;
	void (*sv_func)(int);
};

void	TCPechod(const int), TCPchargend(const int), TCPdaytimed(const int), TCPtimed(const int);

int	passiveTCP(const char *, const int);
int	passiveUDP(const char *);
const char *satop(const struct sockaddr *, char *, socklen_t);
void	doTCP(const struct service *);
void	reaper(int);

struct service svent[] =
	{	{ "echo", TCP_SERV, NOSOCK, TCPechod },
		{ "chargen", TCP_SERV, NOSOCK, TCPchargend },
		{ "daytime", TCP_SERV, NOSOCK, TCPdaytimed },
		{ "time", TCP_SERV, NOSOCK, TCPtimed },
		{ 0, 0, 0, 0 },
	};

#ifndef	MAX
#define	MAX(x, y)	((x) > (y) ? (x) : (y))
#endif	/* MAX */

#define	QLEN		 32

#define	LINELEN		128

extern	unsigned short	portbase;	/* from passivesock()	*/

/*------------------------------------------------------------------------
 * main - Super-server main program
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	struct service	*psv,		/* service table pointer	*/
		*fd2sv[FD_SETSIZE];		/* map fd to service pointer	*/
	int	fd, nfds;
	fd_set	afds, rfds;		/* readable file descriptors	*/

	switch (argc) {
	case 1:
		break;
	case 2:
		portbase = (unsigned short) atoi(argv[1]);
		break;
	default:
		errx(0, "usage: %s [portbase]\n",  argv[0]);
	}

	nfds = 0;
	FD_ZERO(&afds);
	for (psv = &svent[0]; psv->sv_name; ++psv) {
		if (psv->sv_useTCP)
			psv->sv_sock = passiveTCP(psv->sv_name, QLEN);
		else
			psv->sv_sock = passiveUDP(psv->sv_name);
		fd2sv[psv->sv_sock] = psv;
		nfds = MAX(psv->sv_sock+1, nfds);
		FD_SET(psv->sv_sock, &afds);
	}
	(void) signal(SIGCHLD, reaper);

	while (1) {
		memcpy(&rfds, &afds, sizeof rfds);
		if (select(nfds, &rfds, (fd_set *)0, (fd_set *)0,
				(struct timeval *)0) < 0) {
			if (errno == EINTR)
				continue;
			err(1, "select error");
		}
		for (fd=0; fd<nfds; ++fd)
			if (FD_ISSET(fd, &rfds)) {
				psv = fd2sv[fd];
				if (psv->sv_useTCP)
					doTCP(psv);
				else
					psv->sv_func(psv->sv_sock);
			}
	}
}

/*------------------------------------------------------------------------
 * doTCP - handle a TCP service connection request
 *------------------------------------------------------------------------
 */
void
doTCP(const struct service *psv)
{
	struct sockaddr_storage fsa;	/* the from address of a client */
	socklen_t salen;		/* from-address length */
	int	fd, ssock;
    char fmtaddr[128];			/* the formated address string */

	salen = sizeof fsa;
	ssock = accept(psv->sv_sock, (struct sockaddr *)&fsa, &salen);
	if (ssock < 0)
		err(1, "accept failed");

    if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
        err(1, "doTCP() satop failed");
    printf("TCP %s connection from %s\n", psv->sv_name, fmtaddr);

	switch (fork()) {
	case 0:	
		break;
	case -1:
		err(1, "fork failed");
	default:
		(void) close(ssock);
		return;		/* parent */
	}
	/* child */
	for (fd = FD_SETSIZE; fd >= 0; --fd)
		if (fd != ssock)
			(void) close(fd);
	psv->sv_func(ssock);
	exit(0);
}

/*------------------------------------------------------------------------
 * reaper - clean up zombie children
 *------------------------------------------------------------------------
 */
void
reaper(int sig)
{
	int	status;

	while (wait3(&status, WNOHANG, (struct rusage *)0) >= 0)
		/* empty */;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/superd superd.c sv_funcs.c -L../../api -lpassivesock*
 *                                                                       *
 * To run:                                                               *
 * ../../apps/superd                                                     *
 *************************************************************************/