/* sv_funcs.c - TCPechod, TCPchargend, TCPdaytimed, TCPtimed */

#include <netinet/in.h>
#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include <locale.h>
#include <langinfo.h>

#define BUFSIZE 8192 /* max read buffer size	*/

void TCPechod(const int), TCPchargend(const int), TCPdaytimed(const int), TCPtimed(const int);

/*------------------------------------------------------------------------
 * TCPecho - do TCP ECHO on the given socket
 *------------------------------------------------------------------------
 */
void TCPechod(const int sd)
{
	char buf[BUFSIZE];
	int cc;

	while ((cc = read(sd, buf, sizeof buf)))
	{
		if (cc < 0)
			err(1, "echo read failed");
		if (write(sd, buf, cc) != cc)
			err(1, "echo write failed");
	}
}

#define LINELEN 72

/*------------------------------------------------------------------------
 * TCPchargend - do TCP CHARGEN on the given socket
 *------------------------------------------------------------------------
 */
void TCPchargend(const int sd)
{
	char c, buf[LINELEN + 2]; /* print LINELEN chars + \r\n */

	c = ' ';
	buf[LINELEN] = '\r';
	buf[LINELEN + 1] = '\n';
	while (1)
	{
		int i;

		for (i = 0; i < LINELEN; ++i)
		{
			buf[i] = c++;
			if (c > '~')
				c = ' ';
		}
		if (write(sd, buf, LINELEN + 2) != LINELEN + 2)
			break;
	}
}

/*------------------------------------------------------------------------
 * TCPdaytimed - do TCP DAYTIME protocol
 *------------------------------------------------------------------------
 */
void TCPdaytimed(const int sd)
{
	char buf[LINELEN];
	time_t now;

	(void)setlocale(LC_ALL, "");
	(void)time(&now);
	(void)strftime(buf, sizeof buf, nl_langinfo(D_T_FMT), localtime(&now));
	if (write(sd, buf, strlen(buf)) != (ssize_t)strlen(buf))
		err(1, "write failed");
}

#define UNIXEPOCH 2208988800UL /* UNIX epoch, in UCT secs	*/

/*------------------------------------------------------------------------
 * TCPtimed - do TCP TIME protocol
 *------------------------------------------------------------------------
 */
void TCPtimed(const int sd)
{
	time_t now;

	(void)time(&now);
	now = htonl((unsigned long)(now + UNIXEPOCH));
	if (write(sd, &now, sizeof now) != (ssize_t)(sizeof now))
		err(1, "write failed");
}
