/* UDPtime.c - main, UDPtime */

#include <arpa/inet.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include <time.h>
#include <locale.h>
#include <langinfo.h>

#define	UNIXEPOCH	2208988800UL	/* UNIX epoch, in UCT secs	*/

int	UDPtime(const int);
int	connectUDP(const char *, const char *);

/*------------------------------------------------------------------------
 * main - UDP client for TIME service that prints the resulting time
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	const char *host = "localhost";	/* host to use if none supplied	*/
	const char *service = "time";	/* default service name		*/
	int sockd; /* socket descriptor */
	
	switch (argc) {
	case 1:
		break;
	case 3:
		service = argv[2];
		/* FALL THROUGH */
	case 2:
		host = argv[1];
		break;
	default:
		errx(0, "usage: %s [host [port]]\n",  argv[0]);
	}
	
	sockd = connectUDP(host, service);
	exit(UDPtime(sockd));
}

/*------------------------------------------------------------------------
 * UDPtime - do UDP Time on specified socket and print results
 *------------------------------------------------------------------------
 */
int
UDPtime(const int sd)
{
	time_t	now;			/* 32-bit integer to hold time	*/
	ssize_t	nchars; /* receive count */
	char dts[256];	/* buffer for datetime string */

	/* send 1-byte datagram to Time service on specified socket */
	if (write(sd, "", 1) != 1)
		err(1, "UDPtime() write");
		
	/* Read the time */
	if ((nchars = read(sd, (time_t *)&now, sizeof now)) > 0) {
		now = ntohl((unsigned long)now);	/* put in host order	*/
		now -= UNIXEPOCH;		/* convert UCT to UNIX epoch	*/
		//printf("%s", asctime(localtime(&now)) );
		(void)setlocale(LC_ALL, "");
		(void)strftime(dts, sizeof dts, nl_langinfo(D_T_FMT), localtime(&now));
		printf("%s\n", dts);
	}

	if (nchars < 0) err(1, "UDPtime() read");
	return 0;
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/UDPtime UDPtime.c -L../../api -lconnectsock         *
 * To run:                                                               *
 * ../../apps/UDPtime 127.0.0.1                                          *
 * ../../apps/UDPtime 0::1                                               *
 *************************************************************************/