/* UDPtimed.c - UDPtimed, main */

#include <netinet/in.h>
#include <string.h>
#include <time.h>
#include <err.h>
#include <stdio.h>

int	passiveUDP(const char *);
const char *satop(const struct sockaddr *, char *, socklen_t);
void	UDPtimed(const int);

#define	UNIXEPOCH	2208988800UL	/* UNIX epoch, in UCT secs	*/

/*------------------------------------------------------------------------
 * main - Iterative UDP server for TIME service
 *------------------------------------------------------------------------
 */
int
main(int argc, char *argv[])
{
	const char *service = "time";	/* service name or port number	*/
	int	sock;			/* server socket		*/

	switch ( argc ) {
	case	1:
		break;
	case	2:
		service = argv[ 1 ];
		break;
	default:
		errx(0, "usage: %s [port]\n",  argv[0]);
	}

	sock = passiveUDP( service );
	while (1) {
    UDPtimed(sock);
	}
}

/*------------------------------------------------------------------------
 * UDPtimed - do UDP TIME on the given socket
 *------------------------------------------------------------------------
 */

void
UDPtimed(const int sd)
{
	struct sockaddr_storage fsa;	/* the from address of a client	*/
	socklen_t salen;			/* from-address length */
	char	buf[1];			/* "input" buffer; any size > 0	*/
	time_t	now;			/* current time */
	char fmtaddr[128]; 		/* the formated address string */
	ssize_t	nchars; 		/* receive count */

	salen = sizeof fsa;
	if ((nchars = recvfrom(sd, buf, sizeof buf, 0, 
			      (struct sockaddr *)&fsa, &salen)) > 0) {
    	if ((satop((struct sockaddr *)&fsa, fmtaddr, sizeof fmtaddr)) == NULL)
			err(1, "UDPtimed() satop failed");
    	printf("data from %s\n", fmtaddr);

		(void) time(&now);
		now = htonl((unsigned long)(now + UNIXEPOCH));
		if (sendto(sd, &now, sizeof now, 0,
			(struct sockaddr *)&fsa, sizeof fsa) != (ssize_t)(sizeof now))
			err(1, "UDPtimed() sendto");
	}
	if (nchars < 0) err(1, "UDPtimed() recvfrom");
}

/*************************************************************************
 * To compile:                                                           *
 * gcc -Wall -pedantic -std=c11 -g                                       *
 *     -o ../../apps/UDPtimed UDPtimed.c -L../../api -lpassivesock       *
 *                                                                       *
 * To run:                                                               *
 * ../../apps/UDPtimed                                                    *
 *************************************************************************/